// MainDlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "aboutdlg.h"
#include "MainDlg.h"
#include "ResultDlg.h"
#include "mylib/inffile.h"
#include "mylib/processexecutor.h"
#include "SATAControllerInfo.h"
#include "DeviceInfoList.h"
#include "ModelsTargetOSVer.h"
#include "HardwareID.h"
#include "WMIAccess.h"

typedef std::map<std::wstring, ModelsTargetOSVer> ModelsCollection;

namespace fs {
	using namespace boost::filesystem;
}
namespace pt {
	using namespace boost::property_tree;
}

extern fs::wpath const & getModuleDirPath();
extern fs::wpath const * getMainModulePath();
extern int getArchitectureID();

OSVERSIONINFOEX const *getOSVersionInfo();
uint32_t getOSVersion();
std::wstring getOSDisplayString();
std::string runCommandLine(std::wstring const &cmd_line);
bool matchesTargetMoreClosely(
	unsigned int new_architecture_flag, uint32_t new_version,
	unsigned int current_architecture_flag, uint32_t current_version,
	std::wstring const &architecture_str, unsigned int version);
ModelsCollection getModelsSections(
	MyLib::InfFile &inf_file,
	int target_architecture,
	unsigned int version);
std::vector<SATAControllerInfo> enumSATAControllers(GUID const *class_guid);
std::set<std::wstring> enumHardwareIDs(pt::wiptree &wpt, SetNoCase const &tags, std::wstring const &section_name);

BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	return CWindow::IsDialogMessage(pMsg);
}

BOOL CMainDlg::OnIdle()
{
	return false;
}

BOOL CMainDlg::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
	// center the dialog on the screen
	CenterWindow();

	// set icons
	HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
	SetIcon(hIcon, TRUE);
	HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
	SetIcon(hIconSmall, FALSE);

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	UIAddChildWindowContainer(m_hWnd);

	edit01.SubclassWindow(GetDlgItem(IDC_EDIT1));

	try {
		this->displaySATAControllerStatus();
	} catch (MyLib::Exception &e) {
		std::wstringstream ss;
		ss << L"err_code:" << e.getErrorCode() << L"\r\n";
		//ss << L"\r\nerr_msg:" << e.what() << L"\r\n";
		int const *pn = boost::get_error_info<MyLib::errno_info>(e);
		if (pn) {
			ss << *pn << L"\r\n";
		}
		std::wstring const *pm = boost::get_error_info<MyLib::errmsg_info>(e);
		if (pm) {
			ss << *pm << L"\r\n";
		}
		MessageBox(ss.str().c_str(), NULL, MB_ICONSTOP);
	} catch (boost::exception &e) {
		std::string str = boost::diagnostic_information(e);
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (std::exception &e) {
		std::string str = e.what();
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (...) {
		MessageBox(L"s̃G[܂", NULL, MB_ICONSTOP);
	}

	return true;
}

void CMainDlg::OnDestroy()
{
	// unregister message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->RemoveMessageFilter(this);
	pLoop->RemoveIdleHandler(this);
}

void CMainDlg::OnCancel(unsigned int /*uNotifyCode*/, int nID, HWND /*wndCtl*/)
{
	CloseDialog(nID);
}

void CMainDlg::OnInstallINF(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	if (MessageBox(L"foCXhCõCXg[sĂ낵łH", APP_TITLE, MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) != IDYES) {
		return;
	}
	//ResultDlg dlg(this->edit_text);
	//dlg.DoModal();
	//CloseDialog(IDOK);
	OpenFileDialog dlg(NULL, NULL, OFN_HIDEREADONLY | OFN_FILEMUSTEXIST,
		L"INFt@C (*.inf)\0*.inf\0");
	dlg.m_ofn.lpstrTitle = L"foCXhCoINFt@Cw肵ĂB";
	if (dlg.DoModal() == IDOK) {
		try {
			fs::wpath const inf_file_path = dlg.m_szFileName;
			MyLib::InfFile inf_file(inf_file_path.c_str());
			// version
			std::wstring driver_ver, driver_date;
			const MyLib::InfContextIterator inf_end;
			MyLib::InfContextIterator ver_key_it =
				inf_file.findFirstLine(L"Version", L"DriverVer");
			if (ver_key_it != inf_end) {
				unsigned int field_count = ver_key_it->getFieldCount();
				driver_date = ver_key_it->getStringField(1);
				if (field_count >= 2) {
					driver_ver = ver_key_it->getStringField(2);
				}
			}
			// n[hEFAID`FbN
			ModelsCollection models_collection = getModelsSections(inf_file, getArchitectureID(), getOSVersion());
			for (ModelsCollection::const_iterator it = models_collection.begin();
				it != models_collection.end();
				++it) {
					std::wstring models_section = it->first + it->second.getVerString();
					MyLib::InfContextIterator it1 = inf_file.findFirstLine(models_section.c_str());
					MyLib::InfContextIterator const it1_end;
					for (; it1 != it1_end; ++it1) {
						if (it1->getFieldCount() >= 2) {
							std::wstring const inf_hwid_str = it1->getStringField(2);
							HardwareID inf_hwid(inf_hwid_str);
							for (SetNoCase::const_iterator it2 = this->ahci_raid_hwids.begin(); it2 != this->ahci_raid_hwids.end(); ++it2) {
								std::wstring const hwid_str = *it2;
								HardwareID hwid(hwid_str);
								int const hwid_match_level_temp = hwid.match(inf_hwid);
								if (hwid_match_level_temp > 0) {
									// vO{̂̃pX擾
									fs::wpath const &p = *getMainModulePath();
									std::wstring const inf_file_path_str = inf_file_path.wstring();
									std::wstring const cmd_line = L'"' + p.wstring() +L"\" -i \"" + inf_file_path_str + L"\" -h " + hwid_str + L" -b";
									std::string const msg_text = runCommandLine(cmd_line);
									if (msg_text.compare("The inf file has been successfully installed.\r\n") == 0) {
										std::wstring str = L"foCXhCõCXg[܂B\r\n\r\nINFt@C̃pXF\r\n\t"
											+ inf_file_path_str + L"\r\nhCo̍XVF" + driver_date
											+ L"\r\nhCõo[WF" + driver_ver + L"\r\n\r\n" + this->edit_text;
										ResultDlg dlg(str);
										dlg.DoModal();
										CloseDialog(IDOK);
									} else {
										MessageBoxA(this->m_hWnd, msg_text.c_str(), NULL, MB_OK | MB_ICONSTOP);
									}
									return;
								}
							}
						}
					}
			}
		} catch (MyLib::Exception &e) {
			std::wstringstream ss;
			ss << L"err_code:" << e.getErrorCode() << L"\r\n";
			//ss << L"\r\nerr_msg:" << e.what() << L"\r\n";
			int const *pn = boost::get_error_info<MyLib::errno_info>(e);
			if (pn) {
				ss << *pn << L"\r\n";
			}
			std::wstring const *pm = boost::get_error_info<MyLib::errmsg_info>(e);
			if (pm) {
				ss << *pm << L"\r\n";
			}
			MessageBox(ss.str().c_str(), NULL, MB_ICONSTOP);
		} catch (boost::exception &e) {
			std::string str = boost::diagnostic_information(e);
			MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
		} catch (std::exception &e) {
			std::string str = e.what();
			MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
		} catch (...) {
			MessageBox(L"s̃G[܂", NULL, MB_ICONSTOP);
		}
		MessageBox(L"̃foCXhCo͑ΉĂ܂Bʂ̃foCXpAo[WÂ\܂B", NULL, MB_OK | MB_ICONEXCLAMATION);
	}
}

void CMainDlg::OnEnablePCIIDE(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	if (MessageBox(L"WIDEhCo̗LsĂ낵łH", APP_TITLE, MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) != IDYES) {
		return;
	}
	try {
		// vO{̂̃pX擾
		fs::wpath const &p = *getMainModulePath();
		std::wstring const cmd_line = L'"' + p.wstring() + L"\" -e pciide -b";
		std::string const msg_text = runCommandLine(cmd_line);
		if (msg_text.compare("pciide has been enabled.\r\n") == 0) {
			std::wstring str = L"WIDEhCoLɂ܂B\r\n\r\n" + this->edit_text;
			ResultDlg dlg(str);
			dlg.DoModal();
			CloseDialog(IDOK);
		} else {
			MessageBoxA(this->m_hWnd, msg_text.c_str(), NULL, MB_OK | MB_ICONSTOP);
		}
	} catch (MyLib::Exception &e) {
		std::wstringstream ss;
		ss << L"err_code:" << e.getErrorCode() << L"\r\n";
		//ss << L"\r\nerr_msg:" << e.what() << L"\r\n";
		int const *pn = boost::get_error_info<MyLib::errno_info>(e);
		if (pn) {
			ss << *pn << L"\r\n";
		}
		std::wstring const *pm = boost::get_error_info<MyLib::errmsg_info>(e);
		if (pm) {
			ss << *pm << L"\r\n";
		}
		MessageBox(ss.str().c_str(), NULL, MB_ICONSTOP);
	} catch (boost::exception &e) {
		std::string str = boost::diagnostic_information(e);
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (std::exception &e) {
		std::string str = e.what();
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (...) {
		MessageBox(L"s̃G[܂", NULL, MB_ICONSTOP);
	}
}

void CMainDlg::OnEnableMSAHCI(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	if (MessageBox(L"WAHCIhCo̗LsĂ낵łH", APP_TITLE, MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) != IDYES) {
		return;
	}
	try {
		// vO{̂̃pX擾
		fs::wpath const &p = *getMainModulePath();
		std::wstring const cmd_line = L'"' + p.wstring() + L"\" -e msahci -b";
		std::string const msg_text = runCommandLine(cmd_line);
		if (msg_text.compare("msahci has been enabled.\r\n") == 0) {
			std::wstring str = L"WAHCIhCoLɂ܂B\r\n\r\n" + this->edit_text;
			ResultDlg dlg(str);
			dlg.DoModal();
			CloseDialog(IDOK);
		} else {
			MessageBoxA(this->m_hWnd, msg_text.c_str(), NULL, MB_OK | MB_ICONSTOP);
		}
	} catch (MyLib::Exception &e) {
		std::wstringstream ss;
		ss << L"err_code:" << e.getErrorCode() << L"\r\n";
		//ss << L"\r\nerr_msg:" << e.what() << L"\r\n";
		int const *pn = boost::get_error_info<MyLib::errno_info>(e);
		if (pn) {
			ss << *pn << L"\r\n";
		}
		std::wstring const *pm = boost::get_error_info<MyLib::errmsg_info>(e);
		if (pm) {
			ss << *pm << L"\r\n";
		}
		MessageBox(ss.str().c_str(), NULL, MB_ICONSTOP);
	} catch (boost::exception &e) {
		std::string str = boost::diagnostic_information(e);
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (std::exception &e) {
		std::string str = e.what();
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (...) {
		MessageBox(L"s̃G[܂", NULL, MB_ICONSTOP);
	}
}

void CMainDlg::OnEnableIASTORV(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	if (MessageBox(L"WRAIDhCo̗LsĂ낵łH", APP_TITLE, MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) != IDYES) {
		return;
	}
	try {
		// vO{̂̃pX擾
		fs::wpath const &p = *getMainModulePath();
		std::wstring const cmd_line = L'"' + p.wstring() + L"\" -e iastorv -b";
		std::string const msg_text = runCommandLine(cmd_line);
		if (msg_text.compare("iaStorV has been enabled.\r\n") == 0) {
			std::wstring str = L"WRAIDhCoLɂ܂B\r\n\r\n" + this->edit_text;
			ResultDlg dlg(str);
			dlg.DoModal();
			CloseDialog(IDOK);
		} else {
			MessageBoxA(this->m_hWnd, msg_text.c_str(), NULL, MB_OK | MB_ICONSTOP);
		}
	} catch (MyLib::Exception &e) {
		std::wstringstream ss;
		ss << L"err_code:" << e.getErrorCode() << L"\r\n";
		//ss << L"\r\nerr_msg:" << e.what() << L"\r\n";
		int const *pn = boost::get_error_info<MyLib::errno_info>(e);
		if (pn) {
			ss << *pn << L"\r\n";
		}
		std::wstring const *pm = boost::get_error_info<MyLib::errmsg_info>(e);
		if (pm) {
			ss << *pm << L"\r\n";
		}
		MessageBox(ss.str().c_str(), NULL, MB_ICONSTOP);
	} catch (boost::exception &e) {
		std::string str = boost::diagnostic_information(e);
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (std::exception &e) {
		std::string str = e.what();
		MessageBoxA(this->m_hWnd, str.c_str(), NULL, MB_ICONSTOP);
	} catch (...) {
		MessageBox(L"s̃G[܂", NULL, MB_ICONSTOP);
	}
}

void CMainDlg::CloseDialog(int nVal)
{
	DestroyWindow();
	::PostQuitMessage(nVal);
}

void CMainDlg::displaySATAControllerStatus()
{
	// hardwareids.txt ̃pX擾
	fs::wpath const database_file_path = getModuleDirPath()/L"hardwareids.txt";
	if (!fs::exists(database_file_path)) {
		MessageBox(L"hardwareids.txt ܂ł", NULL, MB_ICONERROR);
		this->GetDlgItem(IDC_BTN_INF).EnableWindow(false);
	}
	// hardwareids.txt ǂݍ
	fs::wifstream ifs(database_file_path);
	pt::wiptree wpt;
	pt::read_ini(ifs, wpt);

	std::vector<SATAControllerInfo> sata_controllers = enumSATAControllers(&GUID_DEVCLASS_HDC);
	std::vector<SATAControllerInfo> scsi_sata_controllers = enumSATAControllers(&GUID_DEVCLASS_SCSIADAPTER);
	sata_controllers.insert(sata_controllers.end(), scsi_sata_controllers.begin(), scsi_sata_controllers.end());
	//DeviceInfoList hdc_info_list(&GUID_DEVCLASS_HDC, NULL, DIGCF_PRESENT);
	//for (int i = 0; ; ++i) {
	//	SP_DEVINFO_DATA spdi_data = {sizeof(SP_DEVINFO_DATA)};
	//	if (!::SetupDiEnumDeviceInfo(hdc_info_list.get(), i, &spdi_data)) {
	//		DWORD err_code = ::GetLastError();
	//		if (err_code == ERROR_NO_MORE_ITEMS) {
	//			break;
	//		} else {
	//			BOOST_THROW_EXCEPTION(MyLib::SetupAPIException(HRESULT_FROM_SETUPAPI(err_code)));
	//		}
	//	}
	//	sata_controllers.push_back(spdi_data.DevInst);
	//}
	//DeviceInfoList scsi_info_list(&GUID_DEVCLASS_SCSIADAPTER, NULL, DIGCF_PRESENT);
	//for (int i = 0; ; ++i) {
	//	SP_DEVINFO_DATA spdi_data = {sizeof(SP_DEVINFO_DATA)};
	//	if (!::SetupDiEnumDeviceInfo(scsi_info_list.get(), i, &spdi_data)) {
	//		DWORD err_code = ::GetLastError();
	//		if (err_code == ERROR_NO_MORE_ITEMS) {
	//			break;
	//		} else {
	//			BOOST_THROW_EXCEPTION(MyLib::SetupAPIException(HRESULT_FROM_SETUPAPI(err_code)));
	//		}
	//	}
	//	sata_controllers.push_back(spdi_data.DevInst);
	//}

	std::wstringstream ss;
	HRESULT hr;
	VARIANT vtProp;
	ss << APP_TITLE << L" Ver.0.3 Rev.3\r\n\r\nOSF";// << getOSDisplayString();
	WMIService svc(L"\\\\.\\root\\CIMV2");
	WMIObjectSet const obj_set_end;
	for (WMIObjectSet obj_set = svc.execQuery(L"SELECT * FROM Win32_OperatingSystem"); obj_set != obj_set_end; ++obj_set) {
		WMIObject obj = *obj_set;
		hr = obj.get(L"Caption", &vtProp);	// "Microsoft Windows 7 Enterprise "
		if (!FAILED(hr)) {
			ss << vtProp.bstrVal;
		}
		//hr = obj.get(L"ServicePackMajorVersion", &vtProp);	// 1
		hr = obj.get(L"CSDVersion", &vtProp);	// Service Pack 1
		if (!FAILED(hr)) {
			ss << L" " << vtProp.bstrVal;
		}
		hr = obj.get(L"BuildNumber", &vtProp);	// "7601"
		if (!FAILED(hr)) {
			ss << L" (build " << vtProp.bstrVal << L")";
		}
		hr = obj.get(L"OSArchitecture", &vtProp);	// "64 rbg"
		if (!FAILED(hr)) {
			ss << L" " << vtProp.bstrVal;
		}
	}
	ss << L"\r\n";
	for (WMIObjectSet obj_set = svc.execQuery(L"SELECT * FROM Win32_ComputerSystem"); obj_set != obj_set_end; ++obj_set) {
		WMIObject obj = *obj_set;
		ss << L"Rs[^F";
		hr = obj.get(L"Manufacturer", &vtProp);
		if (!FAILED(hr)) {
			ss << vtProp.bstrVal;
		}
		hr = obj.get(L"Model", &vtProp);
		if (!FAILED(hr)) {
			ss << L" " << vtProp.bstrVal;
		}
		ss << L"\r\n";
	}
	for (WMIObjectSet obj_set = svc.execQuery(L"SELECT * FROM Win32_BaseBoard"); obj_set != obj_set_end; ++obj_set) {
		WMIObject obj = *obj_set;
		ss << L"}U[{[hF";
		hr = obj.get(L"Manufacturer", &vtProp);
		if (!FAILED(hr)) {
			ss << vtProp.bstrVal;
		}
		hr = obj.get(L"Product", &vtProp);
		if (!FAILED(hr)) {
			ss << L" " << vtProp.bstrVal;
		}
		hr = obj.get(L"Version", &vtProp);
		if (!FAILED(hr)) {
			ss << L" " << vtProp.bstrVal;
		}
		ss << L"\r\n";
	}
	for (WMIObjectSet obj_set = svc.execQuery(L"SELECT * FROM Win32_BIOS"); obj_set != obj_set_end; ++obj_set) {
		WMIObject obj = *obj_set;
		hr = obj.get(L"SMBIOSBIOSVersion", &vtProp);
		if (!FAILED(hr)) {
			ss << L"BIOSo[WF" << vtProp.bstrVal << L"\r\n";
		}
	}
	VariantClear(&vtProp);
	//BIOSInformation bios_info;
	//ss << L"Rs[^F" << bios_info.com_sys_vendor << L" " << bios_info.com_sys_model << L"\r\n";
	//ss << L"}U[{[hF" << bios_info.mb_vendor << L" " << bios_info.mb_product << L" " << bios_info.mb_ver << L"\r\n";
	//ss << L"BIOSF" << bios_info.bios_vendor << L" " << bios_info.bios_caption << L" " << bios_info.bios_ver << L"\r\n";
	//ss << L"SMBIOSF" << bios_info.smbios_caption << L" " << bios_info.smbios_major_ver << L"." << bios_info.smbios_minor_ver << L"\r\n\r\n";
	ss << L"\r\n---------- SATARg[ ----------\r\n\r\n";
	SetNoCase tags;
	for (std::vector<SATAControllerInfo>::const_iterator it = sata_controllers.begin(); it != sata_controllers.end(); ++it) {
		SATAControllerInfo const &info = *it;

		bool skip = false;
		std::wstring mode_str;
		std::vector<std::wstring> const compatible_ids = info.getCompatibleIDs();
		for (std::vector<std::wstring>::const_iterator it = compatible_ids.begin(); it != compatible_ids.end(); ++it) {
			if (it->compare(L"PCI\\CC_010601") == 0) {
				mode_str = L"AHCI";
				break;
			} else if (it->compare(L"PCI\\CC_0101") == 0) {
				mode_str = L"IDE";
				break;
			} else if (it->compare(L"PCI\\CC_0104") == 0) {
				mode_str = L"RAID";
				break;
			} else if (it->compare(L"*PNP0600") == 0) {
				skip = true;
				break;
			}
		}
		if (skip) {
			continue;
		}

		std::wstring const display_name = info.getDisplayName();
		std::vector<std::wstring> const hwids = info.getHardwareIDs();

		ss << display_name << L"\r\n샂[hF" << (!mode_str.empty() ? mode_str : L"s") << L"\r\nn[hEFAIDF\r\n";
		for (std::vector<std::wstring>::const_iterator it = hwids.begin(); it != hwids.end(); ++it) {
			ss << L'\t' << *it << L"\r\n";
		}
		ss << L"\r\n";

		if (!mode_str.empty()) {
			pt::wiptree &id = wpt.get_child(mode_str.c_str());
			BOOST_FOREACH(pt::wiptree::value_type const &e, id) {
				// 
				std::wstring tag = e.first.data();
				// E
				std::wstring str2 = e.second.data();
				typedef boost::tokenizer<boost::char_delimiters_separator<wchar_t>,
					std::wstring::const_iterator, std::wstring> wtokenizer;
				wtokenizer tok1(str2, boost::char_delimiters_separator<wchar_t>(true, L"", L", "));
				for (wtokenizer::iterator it1 = tok1.begin(); it1 != tok1.end(); ++it1) {
					for (std::vector<std::wstring>::const_iterator it2 = hwids.begin(); it2 != hwids.end(); ++it2) {
						if (_wcsicmp(it1->c_str(), it2->c_str()) == 0) {
							tags.insert(tag);
							break;
						}
					}
				}
			}
			// SATARg[̃n[hEFAID
			// Ȃ΃{^𖳌
		}
	}
	if (!tags.empty()) {
		std::set<std::wstring> ahci_hwids = enumHardwareIDs(wpt, tags, L"AHCI");
		std::set<std::wstring> raid_hwids = enumHardwareIDs(wpt, tags, L"RAID");
		this->ahci_raid_hwids.insert(ahci_hwids.begin(), ahci_hwids.end());
		this->ahci_raid_hwids.insert(raid_hwids.begin(), raid_hwids.end());
		//// Debug
		//for (SetNoCase::const_iterator it = this->ahci_raid_hwids.begin(); it != this->ahci_raid_hwids.end(); ++it) {
		//	std::wstring str = *it;
		//	ss << str << L"\r\n";
		//}
	} else {
		this->GetDlgItem(IDC_BTN_INF).EnableWindow(false);
	}
	this->edit_text = ss.str();
	edit01.SetWindowText(this->edit_text.c_str());
	//// XN[o[\
	//edit1.ModifyStyle(0, WS_VSCROLL, SWP_DRAWFRAME);

	// vO{̂̃pX擾
	fs::wpath const &p = *getMainModulePath();
	// pciide
	std::string const pciide_msg_text = runCommandLine(L'"' + p.wstring() +L"\" -c pciide");
	if (pciide_msg_text.compare("pciide is enabled.\r\n") == 0) {
		SetDlgItemText(IDC_TEXT_PCIIDE, L"WIDEhCo(pciide)͗LłB");
	} else if (pciide_msg_text.compare("pciide is disabled.\r\n") == 0) {
		SetDlgItemText(IDC_TEXT_PCIIDE, L"WIDEhCo(pciide)͖łB");
		this->GetDlgItem(IDC_BTN_PCIIDE).EnableWindow(true);
	} else {
		// error
		MessageBoxA(NULL, pciide_msg_text.c_str(), NULL, 0);
	}
	if (getOSVersion() >= 0x00060000) {
		// msahci
		std::string const msahci_msg_text = runCommandLine(L'"' + p.wstring() +L"\" -c msahci");
		if (msahci_msg_text.compare("msahci is enabled.\r\n") == 0) {
			SetDlgItemText(IDC_TEXT_MSAHCI, L"WAHCIhCo(msahci)͗LłB");
		} else if (msahci_msg_text.compare("msahci is disabled.\r\n") == 0) {
			SetDlgItemText(IDC_TEXT_MSAHCI, L"WAHCIhCo(msahci)͖łB");
			this->GetDlgItem(IDC_BTN_MSAHCI).EnableWindow(true);
		} else {
			// error
			MessageBoxA(NULL, msahci_msg_text.c_str(), NULL, 0);
		}
		// iaStorV
		std::string const iastorv_msg_text = runCommandLine(L'"' + p.wstring() +L"\" -c iastorv");
		if (iastorv_msg_text.compare("iaStorV is enabled.\r\n") == 0) {
			SetDlgItemText(IDC_TEXT_IASTORV, L"WRAIDhCo(iaStorV)͗LłB");
		} else if (iastorv_msg_text.compare("iaStorV is disabled.\r\n") == 0) {
			SetDlgItemText(IDC_TEXT_IASTORV, L"WRAIDhCo(iaStorV)͖łB");
			this->GetDlgItem(IDC_BTN_IASTORV).EnableWindow(true);
		} else {
			// error
			MessageBoxA(NULL, iastorv_msg_text.c_str(), NULL, 0);
		}
	} else {
		SetDlgItemText(IDC_TEXT_MSAHCI, L"WAHCIhCo͂܂B");
		SetDlgItemText(IDC_TEXT_IASTORV, L"WRAIDhCo͂܂B");
	}
}

/////////////////////////////////////////////////////////////////////////////

OSVERSIONINFOEX const *getOSVersionInfo()
{
	static OSVERSIONINFOEX *os_ver_info_ptr = NULL;
	if (os_ver_info_ptr == NULL) {
		os_ver_info_ptr = new OSVERSIONINFOEX();
		os_ver_info_ptr->dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
		::GetVersionEx(reinterpret_cast<OSVERSIONINFO*>(os_ver_info_ptr));
	}
	return os_ver_info_ptr;
}

uint32_t getOSVersion()
{
	// OS̃o[W`FbN
	OSVERSIONINFOEX const *os_ver_info_ptr = getOSVersionInfo();
	return static_cast<uint32_t>(MAKELONG(os_ver_info_ptr->dwMinorVersion, os_ver_info_ptr->dwMajorVersion));
}

//typedef void (WINAPI *PGNSI)(SYSTEM_INFO*);
//typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, DWORD*);
//
//std::wstring getOSDisplayString()
//{
//	std::wstringstream ss;
//	OSVERSIONINFOEX const *os_ver_info_ptr = getOSVersionInfo();
//	if (os_ver_info_ptr->dwMajorVersion == 5) {
//		if (os_ver_info_ptr->dwMinorVersion == 0) {
//			// 2000
//			ss << L"Windows 2000 ";
//			if (os_ver_info_ptr->wProductType == VER_NT_WORKSTATION) {
//				// Pro
//				ss << L"Professional";
//			} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_DATACENTER) {
//				// Datacenter Server
//				ss << L"Datacenter Server";
//			} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_ENTERPRISE) {
//				// Advanced Server
//				ss << L"Advanced Server";
//			} else {
//				// Server
//				ss << L"Server";
//			}
//		} else if (os_ver_info_ptr->dwMinorVersion == 1) {
//			// XP
//			ss << L"Windows XP ";
//			if (os_ver_info_ptr->wSuiteMask & VER_SUITE_PERSONAL) {
//				// Home
//				ss << L"Home Edition";
//			} else {
//				// Pro
//				ss << L"Professional";
//			}
//		} else if (os_ver_info_ptr->dwMinorVersion == 2) {
//			static PGNSI fnGetNativeSystemInfo = reinterpret_cast<PGNSI>(
//				::GetProcAddress(::GetModuleHandle(L"kernel32.dll"), "GetNativeSystemInfo"));
//			SYSTEM_INFO sys_info;
//			if (fnGetNativeSystemInfo != NULL) {
//				fnGetNativeSystemInfo(&sys_info);
//			} else {
//				GetSystemInfo(&sys_info);
//			}
//			static int const SM_SERVERR2 = 89;
//			if (GetSystemMetrics(SM_SERVERR2)) {
//				// R2
//				ss << L"Windows Server 2003 R2, ";
//			} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_STORAGE_SERVER) {
//				// Windows Storage Server 2003
//				ss << L"Windows Storage Server 2003";
//			} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_WH_SERVER) {
//				// WHS
//				ss << L"Windows Home Server";
//			} else if (os_ver_info_ptr->wProductType == VER_NT_WORKSTATION && sys_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64) {
//				// XP x64
//				ss << L"Windows XP Professional x64 Edition";
//			} else {
//				ss << L"Windows Server 2003, ";
//			}
//			// Test for the server type.
//			if (os_ver_info_ptr->wProductType != VER_NT_WORKSTATION) {
//				if (sys_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64) {
//					if (os_ver_info_ptr->wSuiteMask & VER_SUITE_DATACENTER) {
//						ss << L"Datacenter Edition for Itanium-based Systems";
//					} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_ENTERPRISE) {
//						ss << L"Enterprise Edition for Itanium-based Systems";
//					}
//				} else if (sys_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64) {
//					if (os_ver_info_ptr->wSuiteMask & VER_SUITE_DATACENTER) {
//						ss << L"Datacenter x64 Edition";
//					} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_ENTERPRISE) {
//						ss << L"Enterprise x64 Edition";
//					} else {
//						ss << L"Standard x64 Edition";
//					}
//				} else {
//					if (os_ver_info_ptr->wSuiteMask & VER_SUITE_COMPUTE_SERVER) {
//						ss << L"Compute Cluster Edition";
//					} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_DATACENTER) {
//						ss << L"Datacenter Edition";
//					} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_ENTERPRISE) {
//						ss << L"Enterprise Edition";
//					} else if (os_ver_info_ptr->wSuiteMask & VER_SUITE_BLADE) {
//						ss << L"Web Edition";
//					} else {
//						ss << L"Standard Edition";
//					}
//				}
//			}
//		}
//	} else if (os_ver_info_ptr->dwMajorVersion == 6) {
//		//
//		if (os_ver_info_ptr->dwMinorVersion == 0) {
//			if (os_ver_info_ptr->wProductType == VER_NT_WORKSTATION) {
//				ss << L"Windows Vista ";
//			} else {
//				ss << L"Windows Server 2008 ";
//			}
//		} else if (os_ver_info_ptr->dwMinorVersion == 1) {
//			if (os_ver_info_ptr->wProductType == VER_NT_WORKSTATION) {
//				ss << L"Windows 7 ";
//			} else {
//				ss << L"Windows Server 2008 R2 ";
//			}
//		} else if (os_ver_info_ptr->dwMinorVersion == 2) {
//			if (os_ver_info_ptr->wProductType == VER_NT_WORKSTATION) {
//				ss << L"Windows 8 ";
//			} else {
//				ss << L"Windows Server 2012 ";
//			}
//		}
//
//		static PGPI fnGetProductInfo = reinterpret_cast<PGPI>(
//			::GetProcAddress(::GetModuleHandle(L"kernel32.dll"), "GetProductInfo"));
//		DWORD product_type;
//		fnGetProductInfo(os_ver_info_ptr->dwMajorVersion, os_ver_info_ptr->dwMinorVersion, 0, 0, &product_type);
//		switch(product_type) {
//			case PRODUCT_ULTIMATE:
//				ss << L"Ultimate Edition";
//				break;
//			case PRODUCT_PROFESSIONAL:
//				ss << L"Professional";
//				break;
//			case PRODUCT_HOME_PREMIUM:
//				ss << L"Home Premium Edition";
//				break;
//			case PRODUCT_HOME_BASIC:
//				ss << L"Home Basic Edition";
//				break;
//			case PRODUCT_ENTERPRISE:
//				ss << L"Enterprise Edition";
//				break;
//			case PRODUCT_BUSINESS:
//				ss << L"Business Edition";
//				break;
//			case PRODUCT_STARTER:
//				ss << L"Starter Edition";
//				break;
//			case PRODUCT_CLUSTER_SERVER:
//				ss << L"Cluster Server Edition";
//				break;
//			case PRODUCT_DATACENTER_SERVER:
//				ss << L"Datacenter Edition";
//				break;
//			case PRODUCT_DATACENTER_SERVER_CORE:
//				ss << "Datacenter Edition (core installation)";
//				break;
//			case PRODUCT_ENTERPRISE_SERVER:
//				ss << L"Enterprise Edition";
//				break;
//			case PRODUCT_ENTERPRISE_SERVER_CORE:
//				ss << L"Enterprise Edition (core installation)";
//				break;
//			case PRODUCT_ENTERPRISE_SERVER_IA64:
//				ss << L"Enterprise Edition for Itanium-based Systems";
//				break;
//			case PRODUCT_SMALLBUSINESS_SERVER:
//				ss << L"Small Business Server";
//				break;
//			case PRODUCT_SMALLBUSINESS_SERVER_PREMIUM:
//				ss << L"Small Business Server Premium Edition";
//				break;
//			case PRODUCT_STANDARD_SERVER:
//				ss << L"Standard Edition";
//				break;
//			case PRODUCT_STANDARD_SERVER_CORE:
//				ss << L"Standard Edition (core installation)";
//				break;
//			case PRODUCT_WEB_SERVER:
//				ss << L"Web Server Edition";
//				break;
//		}
//	}
//	// Include service pack (if any) and build number.
//	if (wcslen(os_ver_info_ptr->szCSDVersion) > 0) {
//		ss << L' ' << os_ver_info_ptr->szCSDVersion;
//	}
//	ss << L" (build " << os_ver_info_ptr->dwBuildNumber << L")";
//
//	if (os_ver_info_ptr->dwMajorVersion >= 6) {
//		static PGNSI fnGetNativeSystemInfo = reinterpret_cast<PGNSI>(
//			::GetProcAddress(::GetModuleHandle(L"kernel32.dll"), "GetNativeSystemInfo"));
//		SYSTEM_INFO sys_info;
//		if (fnGetNativeSystemInfo != NULL) {
//			fnGetNativeSystemInfo(&sys_info);
//		} else {
//			GetSystemInfo(&sys_info);
//		}
//		if (sys_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64) {
//			ss << L", 64-bit";
//		} else if (sys_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL) {
//			ss << L", 32-bit";
//		}
//	}
//	ss << L"\r\n";
//	return ss.str();
//}

std::string runCommandLine(std::wstring const &cmd_line)
{
	MyLib::ConsoleProcessExecutor exec;
	exec.run(cmd_line);
	DWORD status;
	std::string str;
	while ((status = exec.getStatus()) != WAIT_ABANDONED) {
		std::vector<char> ret = exec.readStdOut();
		ret.push_back(_T('\0'));
		str += &ret[0];
		if (status == WAIT_OBJECT_0) break;
	}
	return str;
}

bool matchesTargetMoreClosely(
	unsigned int new_architecture_flag, uint32_t new_version,
	unsigned int current_architecture_flag, uint32_t current_version,
	std::wstring const &architecture_str, unsigned int version)
{
	unsigned int this_architecture_flag =
		ModelsTargetOSVer::getArchitectureFlagFromString(architecture_str);
	if ((new_architecture_flag & this_architecture_flag)) {
		if ((current_version == new_version && current_architecture_flag > new_architecture_flag) ||
			(current_version < new_version && new_version <= version)) {
				return true;
		}
	}
	return false;
}

ModelsCollection getModelsSections(MyLib::InfFile &inf_file, int target_architecture, unsigned int version)
{
	std::wstring architecture_str;
	switch (target_architecture) {
	case PROCESSOR_ARCHITECTURE_INTEL:
		architecture_str = _T("X86");
		break;
	case PROCESSOR_ARCHITECTURE_AMD64:
		architecture_str = _T("AMD64");
		break;
	case PROCESSOR_ARCHITECTURE_IA64:
		architecture_str = _T("IA64");
		break;
	default:
		throw std::runtime_error("̃vZbTɂ͑ΉĂ܂");
	}

	ModelsCollection models_collection;
	const MyLib::InfContextIterator inf_end;
	const ModelsTargetOSVer models_no_decorated;
	const unsigned int no_decorated_architecture_flags =
		models_no_decorated.getArchitectureFlag();
	// ^[Qbg CPU Architecture tO (X86, AMD64, IA64)
	const unsigned int target_architecture_flag =
		ModelsTargetOSVer::getArchitectureFlagFromString(architecture_str);

	MyLib::InfContextIterator inf_it = inf_file.findFirstLine(_T("Manufacturer"));
	for (; inf_it != inf_end; ++inf_it) {
		unsigned int field_count = inf_it->getFieldCount();
		// o[WȂǂtĂȂModels Section擾
		std::wstring models_base = inf_it->getStringField(field_count > 0 ? 1 : 0);
		ModelsCollection::iterator it = models_collection.find(models_base);
		if (it == models_collection.end()
			&& (no_decorated_architecture_flags & target_architecture_flag)) {
				it = models_collection.insert(
					std::make_pair(models_base, models_no_decorated)).first;
		}
		// XP(Ver.5.1)ȍ~TargetOSVersionw肳ĂƂ
		if (field_count >= 2 && version >= 0x00050001) {
			// TargetOSVersion(tB[hȍ~)̉
			for (unsigned int i = 2; i <= field_count; ++i) {
				ModelsTargetOSVer new_os_ver(inf_it->getStringField(i));
				// ɋ߂̂
				if (it == models_collection.end()) {
					if (matchesTargetMoreClosely(
						new_os_ver.getArchitectureFlag(),
						new_os_ver.getVersion(),
						models_no_decorated.getArchitectureFlag(),
						models_no_decorated.getVersion(),
						architecture_str,
						version)) {
							it = models_collection.insert(
								std::make_pair(models_base, new_os_ver)).first;
					}
				} else {
					if (matchesTargetMoreClosely(
						new_os_ver.getArchitectureFlag(),
						new_os_ver.getVersion(),
						it->second.getArchitectureFlag(),
						it->second.getVersion(),
						architecture_str,
						version)) {
							it->second = new_os_ver;
					}
				}
			}
		}
	}
	return models_collection;
}

std::vector<SATAControllerInfo> enumSATAControllers(GUID const *class_guid)
{
	std::vector<SATAControllerInfo> sata_controllers;
	DeviceInfoList hdc_info_list(class_guid, NULL, DIGCF_PRESENT);
	for (int i = 0; ; ++i) {
		SP_DEVINFO_DATA spdi_data = {sizeof(SP_DEVINFO_DATA)};
		if (!::SetupDiEnumDeviceInfo(hdc_info_list.get(), i, &spdi_data)) {
			DWORD err_code = ::GetLastError();
			if (err_code == ERROR_NO_MORE_ITEMS) {
				break;
			} else {
				BOOST_THROW_EXCEPTION(MyLib::SetupAPIException(HRESULT_FROM_SETUPAPI(err_code)));
			}
		}
		sata_controllers.push_back(spdi_data.DevInst);
	}
	return sata_controllers;
}

std::set<std::wstring> enumHardwareIDs(pt::wiptree &wpt, SetNoCase const &tags, std::wstring const &section_name)
{
	std::set<std::wstring> ahci_raid_hwids;
	boost::optional<pt::wiptree&> opt = wpt.get_child_optional(section_name);
	if (opt) {
		pt::wiptree &id = *opt;//wpt.get_child(section_name);
		BOOST_FOREACH(pt::wiptree::value_type const &e, id) {
			// 
			std::wstring tag = e.first.data();
			SetNoCase::const_iterator it0 = tags.find(tag);
			if (it0 != tags.end()) {
				// E
				std::wstring str2 = e.second.data();
				typedef boost::tokenizer<boost::char_delimiters_separator<wchar_t>,
					std::wstring::const_iterator, std::wstring> wtokenizer;
				wtokenizer tok1(str2, boost::char_delimiters_separator<wchar_t>(true, L"", L", "));
				for (wtokenizer::iterator it1 = tok1.begin(); it1 != tok1.end(); ++it1) {
					ahci_raid_hwids.insert(*it1);
				}
			}
		}
	}
	return ahci_raid_hwids;
}
