// SATAModeTool.cpp : main source file for SATAModeTool.exe
//

#include "stdafx.h"

#include "resource.h"

#include "aboutdlg.h"
#include "MainDlg.h"

CAppModule _Module;
wchar_t const *APP_TITLE = L"SATA[hύXxc[";

namespace fs {
	using namespace boost::filesystem;
}

int Run(LPTSTR /*lpstrCmdLine*/ = NULL, int nCmdShow = SW_SHOWDEFAULT)
{
	CMessageLoop theLoop;
	_Module.AddMessageLoop(&theLoop);

	CMainDlg dlgMain;

	if(dlgMain.Create(NULL) == NULL)
	{
		ATLTRACE(_T("Main dialog creation failed!\n"));
		return 0;
	}

	dlgMain.ShowWindow(nCmdShow);

	int nRet = theLoop.Run();

	_Module.RemoveMessageLoop();
	return nRet;
}

int getArchitectureID()
{
	typedef void (WINAPI* LPFN_GETNATIVESYSTEMINFO)(SYSTEM_INFO *lpSystemInfo);
	static LPFN_GETNATIVESYSTEMINFO fnGetNativeSystemInfo =
		reinterpret_cast<LPFN_GETNATIVESYSTEMINFO>(::GetProcAddress(
		::GetModuleHandle(L"kernel32.dll"), "GetNativeSystemInfo"));
	if (fnGetNativeSystemInfo != NULL) {
		SYSTEM_INFO info;
		fnGetNativeSystemInfo(&info);
		return info.wProcessorArchitecture;
	}
	return PROCESSOR_ARCHITECTURE_INTEL;
}

fs::wpath const & getModuleDirPath()
{
	static fs::wpath *module_dir_path = NULL;
	if (module_dir_path == NULL) {
		std::vector<wchar_t> buf;
		DWORD buf_count = 128;
		DWORD ret;
		do {
			buf_count <<= 1;
			buf.resize(buf_count);
			ret = ::GetModuleFileName(NULL, &buf[0], buf_count);
		} while (ret == buf_count);
		fs::wpath const module_dir_tmp = &buf[0];
		module_dir_path = new fs::wpath(module_dir_tmp.parent_path());
	}
	return *module_dir_path;
}

fs::wpath const * getMainModulePath()
{
	static fs::wpath *main_module_path = NULL;
	if (main_module_path == NULL) {
		int const architecture_id = getArchitectureID();
		if (architecture_id == PROCESSOR_ARCHITECTURE_INTEL) {
			main_module_path = new fs::wpath(getModuleDirPath()/L"bin\\SATAModeTool_x86.exe");
		} else if (architecture_id == PROCESSOR_ARCHITECTURE_AMD64) {
			main_module_path = new fs::wpath(getModuleDirPath()/L"bin\\SATAModeTool_x64.exe");
		}
	}
	return main_module_path;
}

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPTSTR lpstrCmdLine, int nCmdShow)
{
	fs::wpath const *main_module_path_ptr = getMainModulePath();
	if (main_module_path_ptr == NULL) {
		// Ή
		return EXIT_FAILURE;
	}
	// vO{̂̃pX擾
	if (!fs::exists(*main_module_path_ptr)) {
		std::wstring str = main_module_path_ptr->filename().wstring() + L" ܂łB";
		MessageBox(NULL, str.c_str(), APP_TITLE, MB_ICONERROR);
		return EXIT_FAILURE;
	}
	// Admin
	if (!::IsUserAnAdmin()) {
		::MessageBox(NULL, L"̃\tgEFAgpɂ͊ǗҌKvłB", APP_TITLE, MB_ICONEXCLAMATION);
		return EXIT_FAILURE;
	}

	//HRESULT hRes = ::CoInitialize(NULL);
// If you are running on NT 4.0 or higher you can use the following call instead to 
// make the EXE free threaded. This means that calls come in on a random RPC thread.
	HRESULT hRes = ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
	ATLASSERT(SUCCEEDED(hRes));

	hRes = ::CoInitializeSecurity(
		NULL,
		-1,                          // COM authentication
		NULL,                        // Authentication services
		NULL,                        // Reserved
		RPC_C_AUTHN_LEVEL_DEFAULT,   // Default authentication
		RPC_C_IMP_LEVEL_IMPERSONATE, // Default Impersonation
		NULL,                        // Authentication info
		EOAC_NONE,                   // Additional capabilities
		NULL                         // Reserved
		);
	ATLASSERT(SUCCEEDED(hRes));

	// this resolves ATL window thunking problem when Microsoft Layer for Unicode (MSLU) is used
	::DefWindowProc(NULL, 0, 0, 0L);

	AtlInitCommonControls(ICC_BAR_CLASSES);	// add flags to support other controls

	hRes = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hRes));

	int nRet = Run(lpstrCmdLine, nCmdShow);

	_Module.Term();
	::CoUninitialize();

	return nRet;
}
