#pragma once

//#include "autohandle.h"
//#include "exception.h"

namespace MyLib {
class DriveLayoutInformation
{
	//AutoHandle<INVALID_HANDLE_VALUE> ah;
	std::wstring device_name;
	boost::scoped_array<char> dli_buf;
	size_t dli_size;
	DRIVE_LAYOUT_INFORMATION *dli_ptr;
	int partition_table_entry_count;
	int active_partition;

	static size_t DRIVE_LAYOUT_INFORMATION_SIZE(unsigned int n)
	{
		return sizeof(DRIVE_LAYOUT_INFORMATION) + sizeof(PARTITION_INFORMATION)*(n*4 - 1);
	}

public:
	DriveLayoutInformation(unsigned int device_number)
	: partition_table_entry_count(0)
	, active_partition(-1)
	{
		wchar_t buf[128];
		wsprintf(buf, L"\\\\.\\PhysicalDrive%u", device_number);
		this->device_name = buf;

		MyLib::AutoHandleI h = ::CreateFile(buf,
		                                    GENERIC_READ,
		                                    FILE_SHARE_READ | FILE_SHARE_WRITE,
		                                    NULL,
		                                    OPEN_EXISTING,
		                                    FILE_ATTRIBUTE_NORMAL,
		                                    NULL);
		if (h.isInvalid()) {
			throw MyLib::Win32FileException(buf);
		}

		// fBXÑp[eBV\擾
		unsigned int i = 1;
		this->dli_size = DRIVE_LAYOUT_INFORMATION_SIZE(i);
		this->dli_buf.reset(new char[this->dli_size]);
		this->dli_ptr = reinterpret_cast<DRIVE_LAYOUT_INFORMATION*>(this->dli_buf.get());
		for (;;) {
			DWORD bytes_returned = 0;
			BOOL ret = DeviceIoControl(h.get(),
			                          IOCTL_DISK_GET_DRIVE_LAYOUT,
			                          NULL,
			                          0,
			                          dli_ptr,
			                          this->dli_size,
			                          &bytes_returned,
			                          NULL);
			if (ret) {
				break;
			} else {
				DWORD err_code = ::GetLastError();
				if (err_code == ERROR_INSUFFICIENT_BUFFER) {
					++i;
					this->dli_size = DRIVE_LAYOUT_INFORMATION_SIZE(i);
					this->dli_buf.reset(new char[this->dli_size]);
					this->dli_ptr = reinterpret_cast<DRIVE_LAYOUT_INFORMATION*>(this->dli_buf.get());
				} else {
					throw MyLib::Win32FileException(buf);
				}
			}
		}

		//this->active_partition = -1;
		for (int i = 0; i < std::min(static_cast<int>(dli_ptr->PartitionCount), 4); ++i) {
			if (dli_ptr->PartitionEntry[i].BootIndicator) {
				this->active_partition = i;
			}
			if (dli_ptr->PartitionEntry[i].PartitionType != 0) {
				++(this->partition_table_entry_count);
			}
		}
	}

	unsigned int getPartitionCount() const
	{
		return this->dli_ptr->PartitionCount;
	}

	unsigned int getPartitionTableEntryCount() const
	{
		return this->partition_table_entry_count;
	}

	unsigned int getSignature() const
	{
		return this->dli_ptr->Signature;
	}

	PARTITION_INFORMATION const* getPartitionEntry(unsigned int n) const
	{
		return this->dli_ptr->PartitionEntry + n;
	}

	bool isRecognizedPartition(int partition_index) const
	{
		return IsRecognizedPartition(this->dli_ptr->PartitionEntry[partition_index].PartitionType);
	}

	int getActivePartition() const
	{
		return this->active_partition;
	}

	bool setActivePartiton(int partition_index)
	{
		int ap = this->getActivePartition();
		if (partition_index == ap) {
		 //|| n < 0
		 //|| n >= static_cast<int>(this->getPartitionCount())
		 //|| !IsRecognizedPartition(this->dli_ptr->PartitionEntry[n].PartitionType)) {
			return false;
		}

		MyLib::AutoHandleI h = ::CreateFile(this->device_name.c_str(),
		                                    GENERIC_READ | GENERIC_WRITE,
		                                    FILE_SHARE_READ | FILE_SHARE_WRITE,
		                                    NULL,
		                                    OPEN_EXISTING,
		                                    FILE_ATTRIBUTE_NORMAL,
		                                    NULL);
		if (h.isInvalid()) {
			throw MyLib::Win32FileException(this->device_name);
		}

		boost::scoped_array<char> buf(new char[this->dli_size]);
		std::memcpy(buf.get(), this->dli_ptr, this->dli_size);
		DRIVE_LAYOUT_INFORMATION *ptr = reinterpret_cast<DRIVE_LAYOUT_INFORMATION*>(buf.get());
		if (0 <= ap && ap < 4) {
			ptr->PartitionEntry[ap].BootIndicator = false;
			ptr->PartitionEntry[ap].RewritePartition = true;
		}
		ptr->PartitionEntry[partition_index].BootIndicator = true;
		ptr->PartitionEntry[partition_index].RewritePartition = true;

		DWORD bytes_returned = 0;
		BOOL ret = ::DeviceIoControl(h.get(),
		                             IOCTL_DISK_SET_DRIVE_LAYOUT,
		                             ptr,
		                             static_cast<DWORD>(this->dli_size),
		                             NULL,
		                             0,
		                             &bytes_returned,
		                             NULL);
		if (!ret) {
			throw MyLib::Win32Exception();
		}
		std::memcpy(this->dli_ptr, buf.get(), this->dli_size);
		return true;
	}
};
};  // namespace MyLib
