#pragma once

namespace MyLib {
	typedef boost::error_info<struct tag_errno, int> errno_info;
	typedef boost::error_info<struct tag_errmsg, std::wstring> errmsg_info;

	// ̃G[bZ[WԂ
	inline const std::string makeWin32ErrorText(int error_code)
	{
		HLOCAL buf_ptr;
		uint32_t ret = ::FormatMessageA(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			error_code,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			reinterpret_cast<char*>(&buf_ptr),
			0,
			NULL);
		std::string result;
		if (ret != 0 && buf_ptr != NULL) {
			result = static_cast<const char*>(::LocalLock(buf_ptr));
			::LocalFree(buf_ptr);
		}
		return result;
	}

	// Setup APIpGetLastError()֐
	inline HRESULT getSetupAPILastError()
	{
		DWORD error_code = ::GetLastError();
		return HRESULT_FROM_SETUPAPI(error_code);
	}

    inline const std::string makeStdCErrorText(int errno_value)
    {
#if defined(_MSC_VER) && (_MSC_VER >= 1400) // VC 2005
        char err_text[MAX_PATH];
        err_text[0] = '\0';
        ::strerror_s(err_text, _countof(err_text), errno_value);
#else // others
		const char *err_text = std::strerror(errno_value);
#endif
        return err_text;
    }

	class Exception
		: public boost::exception, public std::runtime_error
	{
	protected:
		int err_code_;

	public:
		Exception(int err_code, std::string const &err_msg)
			: std::runtime_error(err_msg)
			, err_code_(err_code)
		{
		}

		int getErrorCode() const
		{
			return this->err_code_;
		}
	};

	class Win32Exception
		: public Exception
	{
	public:
		Win32Exception(int err_code = ::GetLastError())
			: Exception(err_code, makeWin32ErrorText(err_code))
		{
		}
	};

	class SetupAPIException :
		public Win32Exception
	{
	public:
		SetupAPIException(HRESULT err_code = getSetupAPILastError())
			: Win32Exception(err_code)
		{
		}
	};

	class StdCException
		: public Exception
	{
	public:
		StdCException(int err_code)
			: Exception(err_code, makeStdCErrorText(err_code))
		{
		}
	};
};	// namespace MyLib
